@@ -73,7 +73,7 @@ class StagingClient(object):
         self.infosys = infosys_instance or infosys
 
         try:
-            if isinstance(acopytools, basestring):  # Python 2
+            if isinstance(acopytools, basestring):  # Python 2  # noqa: F821
                 acopytools = {'default': [acopytools]} if acopytools else {}
         except Exception:
             if isinstance(acopytools, str):  # Python 3
@@ -122,7 +122,7 @@ class StagingClient(object):
         :return: default copytools (string).
         """
         try:
-            if isinstance(default_copytools, basestring):  # Python 2
+            if isinstance(default_copytools, basestring):  # Python 2 # noqa: F821
                 default_copytools = [default_copytools] if default_copytools else []
         except Exception:
             if isinstance(default_copytools, str):  # Python 3
@@ -163,7 +163,7 @@ class StagingClient(object):
 
         activities = activities or 'read_lan'
         try:
-            if isinstance(activities, basestring):  # Python 2
+            if isinstance(activities, basestring):  # Python 2  # noqa: F821
                 activities = [activities]
         except Exception:
             if isinstance(activities, str):  # Python 3
@@ -262,6 +262,9 @@ class StagingClient(object):
             query['schemes'] = ['root']
             query['rse_expression'] = 'istape=False\\type=SPECIAL'
 
+        # add signature lifetime for signed URL storages
+        query.update(signature_lifetime=24 * 3600)  # note: default is otherwise 1h
+
         logger.info('calling rucio.list_replicas() with query=%s' % query)
 
         try:
@@ -410,7 +413,7 @@ class StagingClient(object):
             :raise: PilotException in case of controlled error
         """
 
-        raise NotImplemented()
+        raise NotImplementedError()
 
     def transfer(self, files, activity='default', **kwargs):  # noqa: C901
         """
@@ -425,7 +428,7 @@ class StagingClient(object):
         self.trace_report.update(relativeStart=time.time(), transferStart=time.time())
 
         try:
-            if isinstance(activity, basestring):  # Python 2
+            if isinstance(activity, basestring):  # Python 2 # noqa: F821
                 activity = [activity]
         except Exception:
             if isinstance(activity, str):  # Python 3
@@ -972,7 +975,7 @@ class StageOutClient(StagingClient):
             return files
 
         try:
-            if isinstance(activities, (str, unicode)):  # Python 2
+            if isinstance(activities, (str, unicode)):  # Python 2 # noqa: F821
                 activities = [activities]
         except Exception:
             if isinstance(activities, str):  # Python 3
