@@ -5,7 +5,7 @@
 # http://www.apache.org/licenses/LICENSE-2.0
 #
 # Authors:
-# - Paul Nilsson, paul.nilsson@cern.ch, 2017-2019
+# - Paul Nilsson, paul.nilsson@cern.ch, 2017-2021
 # - Wen Guan, wen.guan@cern.ch, 2018
 
 import re
@@ -116,7 +116,7 @@ class ErrorCodes:
     MISSINGINPUTFILE = 1331
     BLACKHOLE = 1332
     NOREMOTESPACE = 1333
-    ATLASSETUPFATAL = 1334
+    SETUPFATAL = 1334
     MISSINGUSERCODE = 1335
     JOBALREADYRUNNING = 1336
     BADMEMORYMONITORJSON = 1337
@@ -145,6 +145,7 @@ class ErrorCodes:
     IMAGENOTFOUND = 1360
     REMOTEFILECOULDNOTBEOPENED = 1361
     XRDCPERROR = 1362
+    KILLPAYLOAD = 1363  # note, not a failure but a kill instruction from Raythena
 
     _error_messages = {
         GENERALERROR: "General pilot error, consult batch log",
@@ -235,12 +236,12 @@ class ErrorCodes:
         NOREPLICAS: "No matching replicas were found in list_replicas() output",
         UNREACHABLENETWORK: "Unable to stage-in file since network is unreachable",
         PAYLOADSIGSEGV: "SIGSEGV: Invalid memory reference or a segmentation fault",
-        NONDETERMINISTICDDM: "Failed to construct SURL for non-deterministic ddm (update AGIS)",
+        NONDETERMINISTICDDM: "Failed to construct SURL for non-deterministic ddm (update CRIC)",
         JSONRETRIEVALTIMEOUT: "JSON retrieval timed out",
         MISSINGINPUTFILE: "Input file is missing in storage element",
         BLACKHOLE: "Black hole detected in file system (consult Pilot log)",
         NOREMOTESPACE: "No space left on device",
-        ATLASSETUPFATAL: "AtlasSetup failed with a fatal exception (consult Payload log)",
+        SETUPFATAL: "Setup failed with a fatal exception (consult Payload log)",
         MISSINGUSERCODE: "User code not available on PanDA server (resubmit task with --useNewCode)",
         JOBALREADYRUNNING: "Job is already running elsewhere",
         BADMEMORYMONITORJSON: "Memory monitor produced bad output",
@@ -268,7 +269,8 @@ class ErrorCodes:
         PANDAQUEUENOTACTIVE: "PanDA queue is not active",
         IMAGENOTFOUND: "Image not found",
         REMOTEFILECOULDNOTBEOPENED: "Remote file could not be opened",
-        XRDCPERROR: "Xrdcp was unable to open file"
+        XRDCPERROR: "Xrdcp was unable to open file",
+        KILLPAYLOAD: "Raythena has decided to kill payload"
     }
 
     put_error_codes = [1135, 1136, 1137, 1141, 1152, 1181]
@@ -332,6 +334,27 @@ class ErrorCodes:
 
         return pilot_error_codes, pilot_error_diags
 
+    def remove_error_code(self, errorcode, pilot_error_codes=[], pilot_error_diags=[]):
+        """
+        Silently remove an error code and its diagnostics from the internal error lists.
+        There is no warning or exception thrown in case the error code is not present in the lists.
+
+        :param errorcode: error code (int).
+        :return: pilot_error_codes, pilot_error_diags
+        """
+
+        if errorcode in pilot_error_codes:
+            try:
+                index = pilot_error_codes.index(errorcode)
+            except ValueError:
+                pass
+            else:
+                # remove the entries in pilot_error_codes and pilot_error_diags
+                pilot_error_codes.pop(index)
+                pilot_error_diags.pop(index)
+
+        return pilot_error_codes, pilot_error_diags
+
     def report_errors(self, pilot_error_codes, pilot_error_diags):
         """
         Report all errors that occurred during running.
