@@ -25,7 +25,7 @@ from pilot.control.job import send_state
 from pilot.util.auxiliary import set_pilot_state, show_memory_usage
 from pilot.util.processes import get_cpu_consumption_time
 from pilot.util.config import config
-from pilot.util.filehandling import read_file, remove_core_dumps
+from pilot.util.filehandling import read_file, remove_core_dumps, get_guid
 from pilot.util.processes import threads_aborted
 from pilot.util.queuehandling import put_in_queue
 from pilot.common.errorcodes import ErrorCodes
@@ -97,11 +97,14 @@ def control(queues, traces, args):
 
 def validate_pre(queues, traces, args):
     """
-    (add description)
+    Get a Job object from the "payloads" queue and validate it.
 
-    :param queues:
-    :param traces:
-    :param args:
+    If the payload is successfully validated (user defined), the Job object is placed in the "validated_payloads" queue,
+    otherwise it is placed in the "failed_payloads" queue.
+
+    :param queues: internal queues for job handling.
+    :param traces: tuple containing internal pilot states.
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
     :return:
     """
     while not args.graceful_stop.is_set():
@@ -174,9 +177,15 @@ def execute_payloads(queues, traces, args):  # noqa: C901
     """
     Execute queued payloads.
 
-    :param queues:
-    :param traces:
-    :param args:
+    Extract a Job object from the "validated_payloads" queue and put it in the "monitored_jobs" queue. The payload
+    stdout/err streams are opened and the pilot state is changed to "starting". A payload executor is selected (for
+    executing a normal job, an event service job or event service merge job). After the payload (or rather its executor)
+    is started, the thread will wait for it to finish and then check for any failures. A successfully completed job is
+    placed in the "finished_payloads" queue, and a failed job will be placed in the "failed_payloads" queue.
+
+    :param queues: internal queues for job handling.
+    :param traces: tuple containing internal pilot states.
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
     :return:
     """
 
@@ -232,6 +241,22 @@ def execute_payloads(queues, traces, args):  # noqa: C901
             out.close()
             err.close()
 
+            pilot_user = os.environ.get('PILOT_USER', 'generic').lower()
+
+            # some HPO jobs will produce new output files (following lfn name pattern), discover those and replace the job.outdata list
+            if job.is_hpo:
+                user = __import__('pilot.user.%s.common' % pilot_user, globals(), locals(), [pilot_user],
+                                  0)  # Python 2/3
+                try:
+                    user.update_output_for_hpo(job)
+                except Exception as e:
+                    logger.warning('exception caught by update_output_for_hpo(): %s' % e)
+                else:
+                    for dat in job.outdata:
+                        if not dat.guid:
+                            dat.guid = get_guid()
+                            logger.warning('guid not set: generated guid=%s for lfn=%s' % (dat.guid, dat.lfn))
+
             #if traces.pilot['nr_jobs'] == 1:
             #    logger.debug('faking job failure in first multi-job')
             #    job.transexitcode = 1
@@ -244,7 +269,6 @@ def execute_payloads(queues, traces, args):  # noqa: C901
             #if job.piloterrorcodes:
             #    exit_code_interpret = 1
             #else:
-            pilot_user = os.environ.get('PILOT_USER', 'generic').lower()
             user = __import__('pilot.user.%s.diagnose' % pilot_user, globals(), locals(), [pilot_user], 0)  # Python 2/3
             try:
                 exit_code_interpret = user.interpret(job)
@@ -388,9 +412,9 @@ def validate_post(queues, traces, args):
     If payload finished correctly, add the job to the data_out queue. If it failed, add it to the data_out queue as
     well but only for log stage-out (in failed_post() below).
 
-    :param queues:
-    :param traces:
-    :param args:
+    :param queues: internal queues for job handling.
+    :param traces: tuple containing internal pilot states.
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
     :return:
     """
 
@@ -422,11 +446,12 @@ def validate_post(queues, traces, args):
 
 def failed_post(queues, traces, args):
     """
-    (add description)
+    Get a Job object from the "failed_payloads" queue. Set the pilot state to "stakeout" and the stageout field to
+    "log", and add the Job object to the "data_out" queue.
 
-    :param queues:
-    :param traces:
-    :param args:
+    :param queues: internal queues for job handling.
+    :param traces: tuple containing internal pilot states.
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
     :return:
     """
 
