@@ -833,10 +833,6 @@ def update_job_data(job):
         if nevents:
             job.nevents = nevents
 
-    # some HPO jobs will produce new output files (following lfn name pattern), discover those and replace the job.outdata list
-    if job.is_hpo:
-        update_output_for_hpo(job)
-
     # extract output files from the job report if required, in case the trf has created additional (overflow) files
     # also make sure all guids are assigned (use job report value if present, otherwise generate the guid)
     if job.metadata and not job.is_eventservice:
@@ -1074,8 +1070,13 @@ def verify_output_files(job):
         # ie job report is ancient / output could not be extracted
         logger.warning('output file list could not be extracted from job report (nothing to verify)')
     else:
-        verified = verify_extracted_output_files(output, lfns_jobdef, job)
+        verified, nevents = verify_extracted_output_files(output, lfns_jobdef, job)
         failed = True if not verified else False
+        if nevents > 0 and not failed and job.nevents == 0:
+            job.nevents = nevents
+            logger.info('number of events from summed up output files: %d' % nevents)
+        else:
+            logger.info('number of events previously set to %d' % job.nevents)
 
     status = True if not failed else False
 
@@ -1090,15 +1091,16 @@ def verify_output_files(job):
 def verify_extracted_output_files(output, lfns_jobdef, job):
     """
     Make sure all output files extracted from the job report are listed.
+    Grab the number of events if possible.
 
     :param output: list of FileSpecs (list).
     :param lfns_jobdef: list of lfns strings from job definition (list).
     :param job: job object.
-    :return: True if successful, False if failed (Boolean)
+    :return: True if successful|False if failed, number of events (Boolean, int)
     """
 
     failed = False
-
+    nevents = 0
     output_jobrep = {}  # {lfn: nentries, ..}
     logger.debug('extracted output file list from job report - make sure all known output files are listed')
 
@@ -1149,10 +1151,12 @@ def verify_extracted_output_files(output, lfns_jobdef, job):
                 remove_from_stageout(lfn, job)
             elif type(nentries) is int and nentries:
                 logger.info('output file %s has %d event(s)' % (lfn, nentries))
+                nevents += nentries
             else:  # should not reach this step
                 logger.warning('case not handled for output file %s with %s event(s) (ignore)' % (lfn, str(nentries)))
 
-    return False if failed else True
+    status = False if failed else True
+    return status, nevents
 
 
 def remove_from_stageout(lfn, job):
@@ -1410,7 +1414,7 @@ def get_db_info(jobreport_dictionary):
 
     db_time = 0
     try:
-        db_data = long(0)  # Python 2
+        db_data = long(0)  # Python 2  # noqa: F821
     except Exception:
         db_data = 0  # Python 3
 
@@ -1446,7 +1450,7 @@ def get_db_info_str(db_time, db_data):
     """
 
     try:
-        zero = long(0)  # Python 2
+        zero = long(0)  # Python 2  # noqa: F821
     except Exception:
         zero = 0  # Python 3
 
@@ -1474,7 +1478,7 @@ def get_cpu_times(jobreport_dictionary):
     """
 
     try:
-        total_cpu_time = long(0)  # Python 2
+        total_cpu_time = long(0)  # Python 2 # noqa: F821
     except Exception:
         total_cpu_time = 0  # Python 3
 
@@ -1507,6 +1511,23 @@ def get_exit_info(jobreport_dictionary):
     return jobreport_dictionary['exitCode'], jobreport_dictionary['exitMsg']
 
 
+def cleanup_looping_payload(workdir):
+    """
+    Run a special cleanup for looping payloads.
+    Remove any root and tmp files.
+
+    :param workdir: working directory (string)
+    :return:
+    """
+
+    for (p, d, f) in os.walk(workdir):
+        for filename in f:
+            if 'pool.root' in filename:
+                path = os.path.join(p, filename)
+                path = os.path.abspath(path)
+                remove(path)
+
+
 def cleanup_payload(workdir, outputfiles=[]):
     """
     Cleanup of payload (specifically AthenaMP) sub directories prior to log file creation.
@@ -1767,9 +1788,12 @@ def remove_redundant_files(workdir, outputfiles=[], islooping=False):
 
     # remove any present user workDir
     path = os.path.join(workdir, 'workDir')
-    if os.path.exists(path) and not islooping:
-        logger.debug('removing \'workDir\' from workdir=%s' % workdir)
-        remove_dir_tree(path)
+    if os.path.exists(path):
+        # remove at least root files from workDir (ie also in the case of looping job)
+        cleanup_looping_payload(path)
+        if not islooping:
+            logger.debug('removing \'workDir\' from workdir=%s' % workdir)
+            remove_dir_tree(path)
 
     # remove additional dirs
     additionals = ['singularity', 'pilot', 'cores']
