@@ -18,7 +18,7 @@ from pilot.common.errorcodes import ErrorCodes
 from pilot.util.auxiliary import set_pilot_state, show_memory_usage
 from pilot.util.config import config
 from pilot.util.container import execute
-from pilot.util.filehandling import get_directory_size, remove_files, get_local_file_size
+from pilot.util.filehandling import get_directory_size, remove_files, get_local_file_size, read_file
 from pilot.util.loopingjob import looping_job
 from pilot.util.math import convert_mb_to_b, human2bytes
 from pilot.util.parameters import convert_to_int, get_maximum_input_sizes
@@ -57,16 +57,7 @@ def job_monitor_tasks(job, mt, args):
         except Exception as e:
             diagnostics = "Exception caught: %s" % e
             logger.warning(diagnostics)
-            import traceback
-            logger.warning(traceback.format_exc())
-            if "Resource temporarily unavailable" in diagnostics:
-                exit_code = errors.RESOURCEUNAVAILABLE
-            elif "No such file or directory" in diagnostics:
-                exit_code = errors.STATFILEPROBLEM
-            elif "No such process" in diagnostics:
-                exit_code = errors.NOSUCHPROCESS
-            else:
-                exit_code = errors.GENERALCPUCALCPROBLEM
+            exit_code = get_exception_error_code(diagnostics)
             return exit_code, diagnostics
         else:
             job.cpuconsumptiontime = int(round(cpuconsumptiontime))
@@ -82,6 +73,14 @@ def job_monitor_tasks(job, mt, args):
         if exit_code != 0:
             return exit_code, diagnostics
 
+        # display OOM process info
+        display_oom_info(job.pid)
+
+    # should the pilot abort the payload?
+    exit_code, diagnostics = should_abort_payload(current_time, mt)
+    if exit_code != 0:
+        return exit_code, diagnostics
+
     # is it time to verify the pilot running time?
 #    exit_code, diagnostics = verify_pilot_running_time(current_time, mt, job)
 #    if exit_code != 0:
@@ -116,6 +115,60 @@ def job_monitor_tasks(job, mt, args):
     return exit_code, diagnostics
 
 
+def display_oom_info(payload_pid):
+    """
+    Display OOM process info.
+
+    :param payload_pid: payload pid (int).
+    """
+
+    payload_score = get_score(payload_pid) if payload_pid else 'UNKNOWN'
+    pilot_score = get_score(os.getpid())
+    logger.info('oom_score(pilot) = %s, oom_score(payload) = %s' % (pilot_score, payload_score))
+
+
+def get_score(pid):
+    """
+    Get the OOM process score.
+
+    :param pid: process id (int).
+    :return: score (string).
+    """
+
+    try:
+        score = '%s' % read_file('/proc/%d/oom_score' % pid)
+    except Exception as e:
+        logger.warning('caught exception reading oom_score: %s' % e)
+        score = 'UNKNOWN'
+    else:
+        if score.endswith('\n'):
+            score = score[:-1]
+
+    return score
+
+
+def get_exception_error_code(diagnostics):
+    """
+    Identify a suitable error code to a given exception.
+
+    :param diagnostics: exception diagnostics (string).
+    :return: exit_code
+    """
+
+    import traceback
+    logger.warning(traceback.format_exc())
+    if "Resource temporarily unavailable" in diagnostics:
+        exit_code = errors.RESOURCEUNAVAILABLE
+    elif "No such file or directory" in diagnostics:
+        exit_code = errors.STATFILEPROBLEM
+    elif "No such process" in diagnostics:
+        exit_code = errors.NOSUCHPROCESS
+    else:
+        exit_code = errors.GENERALCPUCALCPROBLEM
+
+    return exit_code
+
+
 def set_number_used_cores(job):
     """
     Set the number of cores used by the payload.
@@ -168,6 +221,30 @@ def verify_memory_usage(current_time, mt, job):
     return 0, ""
 
 
+def should_abort_payload(current_time, mt):
+    """
+    Should the pilot abort the payload?
+    In the case of Raythena, the Driver is monitoring the time to end jobs and may decide
+    that the pilot should abort the payload. Internally, this is achieved by letting the Actors
+    know it's time to end, and they in turn contacts the pilot by placing a 'pilot_kill_payload' file
+    in the run directory.
+
+    :param current_time: current time at the start of the monitoring loop (int).
+    :param mt: measured time object.
+    :return: exit code (int), error diagnostics (string).
+    """
+
+    # is it time to look for the kill instruction file?
+    killing_time = convert_to_int(config.Pilot.kill_instruction_time, default=600)
+    if current_time - mt.get('ct_kill') > killing_time:
+        path = os.path.join(os.environ.get('PILOT_HOME'), config.Pilot.kill_instruction_filename)
+        if os.path.exists(path):
+            logger.info('pilot encountered payload kill instruction file - will abort payload')
+            return errors.KILLPAYLOAD, ""  # note, this is not an error
+
+    return 0, ""
+
+
 def verify_user_proxy(current_time, mt):
     """
     Verify the user proxy.
@@ -185,7 +262,7 @@ def verify_user_proxy(current_time, mt):
     proxy_verification_time = convert_to_int(config.Pilot.proxy_verification_time, default=600)
     if current_time - mt.get('ct_proxy') > proxy_verification_time:
         # is the proxy still valid?
-        exit_code, diagnostics = userproxy.verify_proxy()
+        exit_code, diagnostics = userproxy.verify_proxy(test=False)  # use test=True to test expired proxy
         if exit_code != 0:
             return exit_code, diagnostics
         else:
@@ -347,12 +424,9 @@ def utility_monitor(job):
         else:  # check the utility output (the selector option adds a substring to the output file name)
             filename = usercommon.get_utility_command_output_filename(utcmd, selector=True)
             path = os.path.join(job.workdir, filename)
-            if os.path.exists(path):
-                logger.info('file: %s exists' % path)
-            else:
+            if not os.path.exists(path):
                 logger.warning('file: %s does not exist' % path)
 
-            # rest
             time.sleep(10)
 
 
